<?php

// autoload_static.php @generated by Composer

namespace Composer\Autoload;

class ComposerStaticInit77e0a56886907409518852c12b239c04
{
    public static $files = array (
        '383eaff206634a77a1be54e64e6459c7' => __DIR__ . '/..' . '/sabre/uri/lib/functions.php',
        '2b9d0f43f9552984cfa82fee95491826' => __DIR__ . '/..' . '/sabre/event/lib/coroutine.php',
        'd81bab31d3feb45bfe2f283ea3c8fdf7' => __DIR__ . '/..' . '/sabre/event/lib/Loop/functions.php',
        'a1cce3d26cc15c00fcd0b3354bd72c88' => __DIR__ . '/..' . '/sabre/event/lib/Promise/functions.php',
        '3569eecfeed3bcf0bad3c998a494ecb8' => __DIR__ . '/..' . '/sabre/xml/lib/Deserializer/functions.php',
        '93aa591bc4ca510c520999e34229ee79' => __DIR__ . '/..' . '/sabre/xml/lib/Serializer/functions.php',
        'ebdb698ed4152ae445614b69b5e4bb6a' => __DIR__ . '/..' . '/sabre/http/lib/functions.php',
        '777dc149d682152f9ab0caaeffcdbb27' => __DIR__ . '/../..' . '/config.php',
        '32a4f751fbcde8755e5f28bd96eed397' => __DIR__ . '/../..' . '/version.php',
        'b05b0d0a4f97d8e7b104a9a31ac2ca03' => UMAPI_PATH . '/mapi.util.php',
        '6a9de07a28f7691abb65b9e10c0d3162' => UMAPI_PATH . '/mapidefs.php',
        'f7a3109aab90144ee6f61ff5d8dc18f8' => UMAPI_PATH . '/mapiguid.php',
        'e6a7d3833b984731fa99a3d51254b714' => UMAPI_PATH . '/mapitags.php',
    );

    public static $prefixLengthsPsr4 = array (
        'g' => 
        array (
            'grommunio\\DAV\\' => 14,
        ),
        'S' => 
        array (
            'Sabre\\Xml\\' => 10,
            'Sabre\\VObject\\' => 14,
            'Sabre\\Uri\\' => 10,
            'Sabre\\HTTP\\' => 11,
            'Sabre\\Event\\' => 12,
            'Sabre\\DAV\\' => 10,
            'Sabre\\DAVACL\\' => 13,
            'Sabre\\CardDAV\\' => 14,
            'Sabre\\CalDAV\\' => 13,
        ),
        'P' => 
        array (
            'Psr\\Log\\' => 8,
        ),
        'M' => 
        array (
            'Monolog\\' => 8,
        ),
    );

    public static $prefixDirsPsr4 = array (
        'grommunio\\DAV\\' => 
        array (
            0 => __DIR__ . '/../..' . '/lib',
        ),
        'Sabre\\Xml\\' => 
        array (
            0 => __DIR__ . '/..' . '/sabre/xml/lib',
        ),
        'Sabre\\VObject\\' => 
        array (
            0 => __DIR__ . '/..' . '/sabre/vobject/lib',
        ),
        'Sabre\\Uri\\' => 
        array (
            0 => __DIR__ . '/..' . '/sabre/uri/lib',
        ),
        'Sabre\\HTTP\\' => 
        array (
            0 => __DIR__ . '/..' . '/sabre/http/lib',
        ),
        'Sabre\\Event\\' => 
        array (
            0 => __DIR__ . '/..' . '/sabre/event/lib',
        ),
        'Sabre\\DAV\\' => 
        array (
            0 => __DIR__ . '/..' . '/sabre/dav/lib/DAV',
        ),
        'Sabre\\DAVACL\\' => 
        array (
            0 => __DIR__ . '/..' . '/sabre/dav/lib/DAVACL',
        ),
        'Sabre\\CardDAV\\' => 
        array (
            0 => __DIR__ . '/..' . '/sabre/dav/lib/CardDAV',
        ),
        'Sabre\\CalDAV\\' => 
        array (
            0 => __DIR__ . '/..' . '/sabre/dav/lib/CalDAV',
        ),
        'Psr\\Log\\' => 
        array (
            0 => __DIR__ . '/..' . '/psr/log/Psr/Log',
        ),
        'Monolog\\' => 
        array (
            0 => __DIR__ . '/..' . '/monolog/monolog/src/Monolog',
        ),
    );

    public static $classMap = array (
        'BaseException' => UMAPI_PATH . '/class.baseexception.php',
        'BaseExceptionTest' => UMAPI_PATH . '/tests/BaseExceptionTest.php',
        'BaseRecurrence' => UMAPI_PATH . '/class.baserecurrence.php',
        'Composer\\InstalledVersions' => __DIR__ . '/..' . '/composer/InstalledVersions.php',
        'ExtendedTokenTest' => UMAPI_PATH . '/tests/ExtendedTokenTest.php',
        'FreeBusy' => UMAPI_PATH . '/class.freebusy.php',
        'FreeBusyTest' => UMAPI_PATH . '/tests/FreeBusyTest.php',
        'KeyCloak' => UMAPI_PATH . '/class.keycloak.php',
        'KeyCloakTest' => UMAPI_PATH . '/tests/KeyCloakTest.php',
        'MAPIException' => UMAPI_PATH . '/class.mapiexception.php',
        'MAPIExceptionTest' => UMAPI_PATH . '/tests/MAPIExceptionTest.php',
        'Meetingrequest' => UMAPI_PATH . '/class.meetingrequest.php',
        'Recurrence' => UMAPI_PATH . '/class.recurrence.php',
        'TaskRecurrence' => UMAPI_PATH . '/class.taskrecurrence.php',
        'TaskRequest' => UMAPI_PATH . '/class.taskrequest.php',
        'Token' => UMAPI_PATH . '/class.token.php',
        'TokenTest' => UMAPI_PATH . '/tests/TokenTest.php',
        'UtilityFunctionsTest' => UMAPI_PATH . '/tests/UtilityFunctionsTest.php',
        'resource' => UMAPI_PATH . '/dev/php-mapi-stub.php',
    );

    public static function getInitializer(ClassLoader $loader)
    {
        return \Closure::bind(function () use ($loader) {
            $loader->prefixLengthsPsr4 = ComposerStaticInit77e0a56886907409518852c12b239c04::$prefixLengthsPsr4;
            $loader->prefixDirsPsr4 = ComposerStaticInit77e0a56886907409518852c12b239c04::$prefixDirsPsr4;
            $loader->classMap = ComposerStaticInit77e0a56886907409518852c12b239c04::$classMap;

        }, null, ClassLoader::class);
    }
}
